#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "stm8s.h"
#include "usrlib-lcd.h"

// clock configrations
// data format {8bit ID , 8bit ADRS , 16bit DATA}
// mark "1" ~~~|_____|~~~~~|___
// mark "0" ~~~|___________|~~~~~|___
//             <--------> 750uS
#define CK_Hz    		16000000  // cpu clk
#define DELAY_us		750       // check "1" or "0" after 750uS

// prototypes
void recieve(void);

// groval variables
char str[20];

void main(void)
{
	CLK_ClockSwitchConfig ( CLK_SWITCHMODE_AUTO, CLK_SOURCE_HSE, DISABLE, CLK_CURRENTCLOCKSTATE_DISABLE );
	// LCD
	LCD_Init();
	// GPIO
	GPIO_Init(GPIOA, GPIO_PIN_5, GPIO_MODE_IN_PU_IT); // IRDA interrupt
	GPIO_Init(GPIOD, GPIO_PIN_0, GPIO_MODE_OUT_PP_HIGH_FAST); // LED indicator
	// GPIO interrupt
	EXTI_DeInit();
	EXTI_SetExtIntSensitivity( EXTI_PORT_GPIOA, EXTI_SENSITIVITY_FALL_ONLY );
	// TIM4 interrupt
	TIM4_DeInit();
	TIM4_TimeBaseInit( TIM4_PRESCALER_16, 1000 ); // 16MHz/16/1000=1kHz
	TIM4_ITConfig(TIM4_IT_UPDATE, ENABLE);
	TIM4_Cmd(ENABLE);
	
  enableInterrupts();
	while(1) { recieve(); } // Main loop
}

// IRDA negative edge interrupt
void gpioa(void) interrupt 3 {
	if(GPIO_ReadInputPin( GPIOA, GPIO_PIN_5 )==0) {
		// Delay timer start
		TIM2_DeInit();
		TIM2_TimeBaseInit( TIM2_PRESCALER_1, (u16)((float)CK_Hz/(float)1000000*(float)DELAY_us-1) );
		TIM2_ITConfig(TIM2_IT_UPDATE, ENABLE);
		TIM2_ClearFlag(TIM2_FLAG_UPDATE);
		TIM2_Cmd(ENABLE);
		// timeout timer start
		//GPIO_WriteHigh( GPIOD, GPIO_PIN_0 );
		TIM1_DeInit();
		TIM1_TimeBaseInit( 0, TIM1_COUNTERMODE_UP, 0xFFFE, 0 );
		TIM1_ITConfig(TIM1_IT_UPDATE, ENABLE);
		TIM1_ClearFlag(TIM1_FLAG_UPDATE);
		TIM1_Cmd(ENABLE);
		return;
	}
}

// timeout timer
u8 timeout=0;
void Tim1Update(void) interrupt 11
{
	//GPIO_WriteLow( GPIOD, GPIO_PIN_0 );
	TIM1_Cmd(DISABLE);
	timeout=1;  // recieve timeout
	TIM1_ClearFlag(TIM1_FLAG_UPDATE);
	return;
}

// sample recieved data
u8 recieved_bit_exist=0;
u32 recieved_bit=0;
void Tim2Update(void) interrupt 13
{
	//GPIO_WriteHigh ( GPIOD, GPIO_PIN_0 ); // LED indicator
	//GPIO_WriteLow ( GPIOD, GPIO_PIN_0 ); // LED indicator
	//GPIO_WriteReverse ( GPIOD, GPIO_PIN_0 ); // LED indicator
	if(GPIO_ReadInputPin(GPIOA,GPIO_PIN_5)) {
		recieved_bit=1;
		//GPIO_WriteHigh ( GPIOD, GPIO_PIN_0 );
	}
	else {
		recieved_bit=0;
		//GPIO_WriteLow ( GPIOD, GPIO_PIN_0 );
	}
	recieved_bit_exist=1;
	TIM2_Cmd(DISABLE);
	TIM2_ClearFlag(TIM2_FLAG_UPDATE);
	return;
}

// 1kHz chopper LED indicator
u16 cnt_tim4=0;
u16 comp_tim4;
void TIM4_interrupt(void) interrupt 23  {
	cnt_tim4+=2000;
	if(cnt_tim4>comp_tim4) GPIO_WriteHigh( GPIOD, GPIO_PIN_0 );
	else GPIO_WriteLow( GPIOD, GPIO_PIN_0 );
	TIM4_ClearFlag(TIM4_FLAG_UPDATE);  
}

//*************************************************
//    recieve routine
//*************************************************
// recieve device parameter
u8 id_self = 0x32;  // device ID of this device
u32 rdat,rdat1,rdat2,rdat3; // recieved data consists of ID+ADRS+DATA
u8 seq=0;
u16 id=0,adrs=0,dat=0;
void recieve(void)
{
	if(timeout) { timeout=0; seq=0; rdat=0; } // wait next data sequence
	else {
		switch(seq) {
			case(0):  if(recieved_bit_exist){rdat+=recieved_bit<<31; recieved_bit_exist=0; seq++;} break;
			case(1):  if(recieved_bit_exist){rdat+=recieved_bit<<30; recieved_bit_exist=0; seq++;} break;
			case(2):  if(recieved_bit_exist){rdat+=recieved_bit<<29; recieved_bit_exist=0; seq++;} break;
			case(3):  if(recieved_bit_exist){rdat+=recieved_bit<<28; recieved_bit_exist=0; seq++;} break;
			case(4):  if(recieved_bit_exist){rdat+=recieved_bit<<27; recieved_bit_exist=0; seq++;} break;
			case(5):  if(recieved_bit_exist){rdat+=recieved_bit<<26; recieved_bit_exist=0; seq++;} break;
			case(6):  if(recieved_bit_exist){rdat+=recieved_bit<<25; recieved_bit_exist=0; seq++;} break;
			case(7):  if(recieved_bit_exist){rdat+=recieved_bit<<24; recieved_bit_exist=0; seq++;} break;
			case(8):  if(recieved_bit_exist){rdat+=recieved_bit<<23; recieved_bit_exist=0; seq++;} break;
			case(9):  if(recieved_bit_exist){rdat+=recieved_bit<<22; recieved_bit_exist=0; seq++;} break;
			case(10): if(recieved_bit_exist){rdat+=recieved_bit<<21; recieved_bit_exist=0; seq++;} break;
			case(11): if(recieved_bit_exist){rdat+=recieved_bit<<20; recieved_bit_exist=0; seq++;} break;
			case(12): if(recieved_bit_exist){rdat+=recieved_bit<<19; recieved_bit_exist=0; seq++;} break;
			case(13): if(recieved_bit_exist){rdat+=recieved_bit<<18; recieved_bit_exist=0; seq++;} break;
			case(14): if(recieved_bit_exist){rdat+=recieved_bit<<17; recieved_bit_exist=0; seq++;} break;
			case(15): if(recieved_bit_exist){rdat+=recieved_bit<<16; recieved_bit_exist=0; seq++;} break;
			case(16): if(recieved_bit_exist){rdat+=recieved_bit<<15; recieved_bit_exist=0; seq++;} break;
			case(17): if(recieved_bit_exist){rdat+=recieved_bit<<14; recieved_bit_exist=0; seq++;} break;
			case(18): if(recieved_bit_exist){rdat+=recieved_bit<<13; recieved_bit_exist=0; seq++;} break;
			case(19): if(recieved_bit_exist){rdat+=recieved_bit<<12; recieved_bit_exist=0; seq++;} break;
			case(20): if(recieved_bit_exist){rdat+=recieved_bit<<11; recieved_bit_exist=0; seq++;} break;
			case(21): if(recieved_bit_exist){rdat+=recieved_bit<<10; recieved_bit_exist=0; seq++;} break;
			case(22): if(recieved_bit_exist){rdat+=recieved_bit<<9;  recieved_bit_exist=0; seq++;} break;
			case(23): if(recieved_bit_exist){rdat+=recieved_bit<<8;  recieved_bit_exist=0; seq++;} break;
			case(24): if(recieved_bit_exist){rdat+=recieved_bit<<7;  recieved_bit_exist=0; seq++;} break;
			case(25): if(recieved_bit_exist){rdat+=recieved_bit<<6;  recieved_bit_exist=0; seq++;} break;
			case(26): if(recieved_bit_exist){rdat+=recieved_bit<<5;  recieved_bit_exist=0; seq++;} break;
			case(27): if(recieved_bit_exist){rdat+=recieved_bit<<4;  recieved_bit_exist=0; seq++;} break;
			case(28): if(recieved_bit_exist){rdat+=recieved_bit<<3;  recieved_bit_exist=0; seq++;} break;
			case(29): if(recieved_bit_exist){rdat+=recieved_bit<<2;  recieved_bit_exist=0; seq++;} break;
			case(30): if(recieved_bit_exist){rdat+=recieved_bit<<1;  recieved_bit_exist=0; seq++;} break;
			case(31): if(recieved_bit_exist){rdat+=recieved_bit;     recieved_bit_exist=0; seq=40;} break;

			case(40): // display recieved data
								id		=(rdat&0xFF000000)>>24;
								adrs	=(rdat&0x00FF0000)>>16;
								dat		=	rdat&0x0000FFFF;
								sprintf(str,"myID%02x",id_self);
								LCD_Disp_Str(str,LCD_DISP_UPPERLINE);
								sprintf(str,"ID%02x A:%02x D:%04x",id,adrs,dat);
								LCD_Disp_Str(str,LCD_DISP_LOWERLINE);
								seq=41;
								break;
			case(41): rdat3=rdat2; rdat2=rdat1; rdat1=rdat; seq=42; break;
			case(42): // check error or good (3 times the same?)
								if(rdat3==rdat2 && rdat2==rdat1) seq=50; else seq=0; break;

			case(50): // ID check
								id		=(rdat3&0xFF000000)>>24;
								adrs	=(rdat3&0x00FF0000)>>16;
								dat		=	rdat3&0x0000FFFF;
								if(id_self!=id) seq=0; // not myself
								else seq=51;
								break;
			case(51): comp_tim4=dat; // GOOD data process
								seq=0;
								break;
			default: 	seq=0;
								break;
		}
	}
	return;
}

#ifdef USE_FULL_ASSERT
void assert_failed(uint8_t* file, uint32_t line)
{ 
  while (1)  {  }
}
#endif
