#include "stm8s.h"

// clock configrations
// data format {8bit ID , 8bit ADRS , 16bit DATA}
// mark "1" ~~~|_____|~~~~~|___
// mark "0" ~~~|___________|~~~~~|___
// symbol _____|||||_________|||||_____
#define CK_Hz    		16000000  // cpu clk
#define WIDTH1_us		470       // mark "1" 500uS
#define WIDTH0_us		970      // mark "0" 1000uS
#define CARRIER_Hz  38000     // carrier freq 38kHz
#define TIM2_ARR 		((float)CK_Hz/(float)CARRIER_Hz-1)

// prototypes
void transmit(void);

void main(void)
{
	CLK_ClockSwitchConfig ( CLK_SWITCHMODE_AUTO, CLK_SOURCE_HSE, DISABLE, CLK_CURRENTCLOCKSTATE_DISABLE );

	GPIO_Init(GPIOD, GPIO_PIN_0, GPIO_MODE_OUT_PP_HIGH_FAST); // LED indicator

	// TIM1 is data send interval
	// TIM1 is LED indicator
	TIM1_DeInit();
	TIM1_TimeBaseInit( 16000, TIM1_COUNTERMODE_UP, 166, 0 ); // 16MHz/16000/166=6Hz
	TIM1_ITConfig(TIM1_IT_UPDATE, ENABLE);
	TIM1_Cmd(ENABLE);

	// TIM2 is 38kHz carrier frequency
	TIM2_DeInit();
	TIM2_TimeBaseInit( TIM2_PRESCALER_1, (u16)TIM2_ARR-1);  // 16MHz/TIM2_ARR=38kHz
	TIM2_OC1Init(TIM2_OCMODE_PWM1, TIM2_OUTPUTSTATE_ENABLE, (u16)(TIM2_ARR/2), TIM2_OCPOLARITY_HIGH); // output D4
	TIM2_Cmd(ENABLE);

	// TIM3 is bit timer

  enableInterrupts();

	while(1) { transmit(); } // Main loop
}

// data send interval timer
u8 start=0;
void Tim1Update(void) interrupt 11
{
	start=1;  // transfer start
	GPIO_WriteReverse ( GPIOD, GPIO_PIN_0 ); // LED indicator
	TIM1_ClearFlag(TIM1_FLAG_UPDATE);
	return;
}

u8 end_of_tim3_delay;
#define SHORT 1
#define LONG  2
void delay(u8 x){ // x=1,0
	if(x==SHORT) { // mark "1"
		// Delay timer interrupt
		TIM3_DeInit();
		TIM3_TimeBaseInit( TIM3_PRESCALER_1, (u16)((float)CK_Hz/(float)1000000*(float)WIDTH1_us-1) );
		TIM3_ITConfig(TIM3_IT_UPDATE, ENABLE);
		TIM3_ClearFlag(TIM3_FLAG_UPDATE);
		TIM3_Cmd(ENABLE);
	}
	else if(x==LONG){ // mark "0"
		// Delay timer interrupt
		TIM3_DeInit();
		TIM3_TimeBaseInit( TIM3_PRESCALER_1, (u16)((float)CK_Hz/(float)1000000*(float)WIDTH0_us-1) );
		TIM3_ITConfig(TIM3_IT_UPDATE, ENABLE);
		TIM3_ClearFlag(TIM3_FLAG_UPDATE);
		TIM3_Cmd(ENABLE);
	}
	// wait untill tim3 end
	while(end_of_tim3_delay!=1){;}
	end_of_tim3_delay=0;
	return;
}

void Tim3Update(void) interrupt 15
{
	TIM3_Cmd(DISABLE);
	end_of_tim3_delay=1;
	TIM3_ClearFlag(TIM3_FLAG_UPDATE);
	return;
}

//*************************************************
//    transmit routine
//*************************************************

// mark "1" ___||||||_____||||
// mark "0" ___|||||||||||_____||||
void mark(u8 x){ // x=0,1
	TIM2_Cmd(ENABLE);
	if(x==1) delay(SHORT);
	else if(x==0) delay(LONG);
	TIM2_Cmd(DISABLE); delay(SHORT);
}

// recieve device parameter
u8 id = 0x32;  // recieve device ID
u8 adrs = 0xf0; // recievr device ADDRESS
u16 dat = 0x0000; // DATA
u8 seq=0;
u8 times=0;
void transmit(void)
{
	switch(seq){
		case(0): if(start){start=0;seq=1;}  break; // transmit start 1st
		case(1): dat=dat+0x1234; seq=2; break; // data change
		case(2): times=1; seq=11; break;
		
		case(5): if(start){start=0;times=2;seq=11;}  break; // transmit start 2nd

		case(6): if(start){start=0;times=3;seq=11;}  break; // transmit start 3rd

		case(11): mark((id&0x80)>>7); seq++; break; // transmit ID
		case(12): mark((id&0x40)>>6); seq++; break;
		case(13): mark((id&0x20)>>5); seq++; break;
		case(14): mark((id&0x10)>>4); seq++; break;
		case(15): mark((id&0x08)>>3); seq++; break;
		case(16): mark((id&0x04)>>2); seq++; break;
		case(17): mark((id&0x02)>>1); seq++; break;
		case(18): mark((id&0x01));    seq=20; break;
		
		case(20): mark((adrs&0x80)>>7); seq++; break; // transmit ADRS
		case(21): mark((adrs&0x40)>>6); seq++; break;
		case(22): mark((adrs&0x20)>>5); seq++; break;
		case(23): mark((adrs&0x10)>>4); seq++; break;
		case(24): mark((adrs&0x08)>>3); seq++; break;
		case(25): mark((adrs&0x04)>>2); seq++; break;
		case(26): mark((adrs&0x02)>>1); seq++; break;
		case(27): mark((adrs&0x01)); seq=30; break;
		
		case(30): mark((dat&0x8000)>>15); seq++; break; // transmit DATA
		case(31): mark((dat&0x4000)>>14); seq++; break;
		case(32): mark((dat&0x2000)>>13); seq++; break;
		case(33): mark((dat&0x1000)>>12); seq++; break;
		case(34): mark((dat&0x0800)>>11); seq++; break;
		case(35): mark((dat&0x0400)>>10); seq++; break;
		case(36): mark((dat&0x0200)>>9); seq++; break;
		case(37): mark((dat&0x0100)>>8); seq++; break;
		case(38): mark((dat&0x0080)>>7); seq++; break;
		case(39): mark((dat&0x0040)>>6); seq++; break;
		case(40): mark((dat&0x0020)>>5); seq++; break;
		case(41): mark((dat&0x0010)>>4); seq++; break;
		case(42): mark((dat&0x0008)>>3); seq++; break;
		case(43): mark((dat&0x0004)>>2); seq++; break;
		case(44): mark((dat&0x0002)>>1); seq++; break;
		case(45): mark((dat&0x0001)); seq=50; break;
		
		case(50): if(times==1) seq=5; // complete 1st transmit
							else if(times==2) seq=6; // complete 2nd transmit
							else if(times>=3) seq=0; // complete 3rd transmit
							break;
	}
	return;
}

#ifdef USE_FULL_ASSERT
void assert_failed(uint8_t* file, uint32_t line)
{ 
  while (1)  {  }
}
#endif
